/* ============================================================ *
 * decomp_eb_demo.c						*
 * Martin Kilbinger, Liping Fu 2009				*
 * ============================================================ */

#include <stdio.h>
#include <stdlib.h>

#include "errorlist.h"
#include "io.h"
#include "maths.h"
#include "cosmo.h"
#include "lensing.h"
#include "nofz.h"
#include "decomp_eb.h"


#define N 6
int main(int argc, char** argv)
{
   error *myerr = NULL, **err;
   double *a, x, eta, R, tp, tm, Psi, rp, rm;
   char choice;
   cosmo_lens *model;
   FILE *F;

   /* === CFHTLS Wide 3rd data release === */

   /* S/N, Psi=19' eta=1/50 */
   double a0[N] = {0.1197730890, -0.3881211865, 0.5212557875, -0.3440507036, 0.2761305382, -0.07286690971};
   /* FoM, Psi=222', eta=1/10 */
   double a1[N] = {0.009877788826, 0.1061397843, -0.4300211814, 0.5451016406, -0.3372272549, 0.1716983151};
   /* FoM, Psi=222', eta=1/50 */
   double a2[N] = {0.1239456383, -0.3881431858, 0.5579593467, -0.3679282338, 0.1540941993, 0.01293361618};

   err = &myerr;

   printf("decomp_eb_demo (M. Kilbinger, L. Fu 2009)\n");

   printf("Choice of coefficients for filter function:\n");
   printf("  Covariance from CFHTLS Wide 3rd data release (Fu et al. 2008)\n");
   printf("     S/N, eta=1/50          [0]\n");
   printf("     FoM, eta=1/10          [1]\n");
   printf("     FoM, eta=1/50          [2]\n");
   printf("Choice? ");

   choice = getchar();
   switch (choice) {
      case '0' : a = a0; eta = 1.0/50.0; break;
      case '1' : a = a1; eta = 1.0/10.0; break;
      case '2' : a = a2; eta = 1.0/50.0; break;
      default  : assert(0);
   }

   R = (1.0+eta)/(1.0-eta);    /* FK09 (10) */

   printf("Writing filter functions T+, T- to file 'Tpm'\n");
   F = fopen_err("Tpm", "w", err);
   exitOnError(*err, stderr);
   for (x=-1.0; x<=1.0005; x+=0.005) {
      tp = Tp(x, a, N, cheby2, err);
      exitOnError(*err, stderr);
      tm = Tm(x, a, N, cheby2, R, err);
      exitOnError(*err, stderr);
      fprintf(F, "% f % f % f\n", x, tp, tm);
   }
   fclose(F);

   F = fopen_err("cosmo_lens.par", "r", err);
   exitOnError(*err, stderr);
   read_cosmological_parameters_lens(&model, F, err);
   exitOnError(*err, stderr);
   fclose(F);

   printf("Writing shear functions R_E, R_B to file 'REB'\n");
   F = fopen_err("REB", "w", err);
   exitOnError(*err, stderr);
   for (Psi=1.0*arcmin; Psi<=250*arcmin; Psi*=1.2) {
      rp = RR(model, Psi*eta, Psi, a, N, cheby2, comb_none, +1, err);
      exitOnError(*err, stderr);
      rm = RR(model, Psi*eta, Psi, a, N, cheby2, comb_none, -1, err);
      exitOnError(*err, stderr);
      fprintf(F, "% f % g % g\n", Psi/arcmin, 0.5*fabs(rp+rm), 0.5*fabs(rp-rm));
      fflush(F);
   }
   fclose(F);


   return 0;
}
