/* ============================================================ *
 * lensingdemo.c						*
 * Martin Kilbinger 2008					*
 * ============================================================ */

#include <stdio.h>
#include <stdlib.h>

#include "errorlist.h"
#include "io.h"
#include "maths.h"
#include "cosmo.h"
#include "cmb_bao.h"
#include "lensing.h"
#include "nofz.h"


/* === Redshift distribution, one redshift bin (no tomography === */
#define __FUNC__ "set_redshift_distribution_single"
int set_redshift_distribution_single(int **NNZ, nofz_t **NOFZ, double **PAR_NZ, error **err)
{
   int Nzbin;
   int *Nnz;
   nofz_t *nofz;
   double *par_nz;

   /* One redshift bin */
   Nzbin = 1;

   Nnz   = malloc_err(Nzbin*sizeof(int), err);             forwardError(*err, __LINE__, 0);
   nofz  = malloc_err(Nzbin*sizeof(int), err);             forwardError(*err, __LINE__, 0);

   // Hamana's (old) simulations, single redshift plane
   //Nnz[0] = 5;
   //par_nz = malloc_err(sizeof(double)*Nnz[0]*Nzbin, err); forwardError(*err, __LINE__, 0);
   //par_nz[0] = par_nz[1] = par_nz[2] = 0.9772;
   //nofz[0] = single;

   // Fu et al. (2008)
   Nnz[0] = 5;
   par_nz = malloc_err(sizeof(double)*Nnz[0]*Nzbin, err);     forwardError(*err, __LINE__, 0);
   par_nz[0] = 0.0; par_nz[1] = 6.0; par_nz[2] = 0.61; par_nz[3] = 8.13, par_nz[4] = 0.62;
   nofz[0] = ymmk;

   /* Histogram */
   //Nnz[0] = 9;
   //par_nz = malloc_err(sizeof(double)*Nnz[0]*Nzbin, err);       forwardError(*err, __LINE__, 0);
   //par_nz[0] = 0.2;  par_nz[2] = 0.4;  par_nz[3] = 0.6;  par_nz[4] = 0.8; par_nz[1] = 1.0;
   //par_nz[5] = 1.0;  par_nz[6] = 3.0;  par_nz[7] = 4.3;  par_nz[8] = 2.5;
   //nofz[0] = hist;

   /* Histogram from file */
   //par_nz = read_par_nz_hist("nofz.hist", Nnz, err);
   //forwardError(*err, __LINE__, 0);
   //nofz[0] = hist;

   /* Copy allocated pointer addresses */
   *NNZ    = Nnz;
   *NOFZ   = nofz;
   *PAR_NZ = par_nz;

   return Nzbin;
}
#undef __FUNC__

/* === Redshift distribution with tomography === */
#define __FUNC__ "set_redshift_distribution_tomo"
int set_redshift_distribution_tomo(int **NNZ, nofz_t **NOFZ, double **PAR_NZ, error **err)
{
   int Nzbin;
   int *Nnz;
   nofz_t *nofz;
   double *par_nz;

   Nzbin = 2;

   Nnz   = malloc_err(Nzbin*sizeof(int), err);             forwardError(*err, __LINE__, 0);
   nofz  = malloc_err(Nzbin*sizeof(int), err);             forwardError(*err, __LINE__, 0);

   // Exponential law
   Nnz[0] = Nnz[1] = 5;
   nofz[0] = nofz[1] = ludo;
   par_nz = malloc_err(sizeof(double)*Nnz[0]*Nzbin, err); forwardError(*err, __LINE__, 0);
   par_nz[0]   = 0.2;         // zmin
   par_nz[1]   = 0.6;         // zmax
   par_nz[2]   = 2.0;         // alpha
   par_nz[3]   = 1.5;         // beta
   par_nz[4]   = 0.45;        // z0
   par_nz[Nnz[0]+0] = 0.5;
   par_nz[Nnz[0]+1] = 1.0;
   par_nz[Nnz[0]+2] = 2.0;
   par_nz[Nnz[0]+3] = 1.5;
   par_nz[Nnz[0]+4] = 0.7;

   // Single redshift plane
   /*
   Nnz[0] = Nnz[1] = 3;
   nofz[0] = nofz[1] = single;
   par_nz = malloc_err(sizeof(double)*Nnz[0]*Nzbin, err); forwardError(*err, __LINE__, 0);
   par_nz[0] = par_nz[1] = par_nz[2] = 0.5;
   */

   /* Histogram from file */
   //par_nz = read_par_nz_hist("nofz.hist", Nnz, err);
   //forwardError(*err, __LINE__, 0);
   //nofz[0] = hist;

   /* Copy allocated pointer addresses */
   *NNZ    = Nnz;
   *NOFZ   = nofz;
   *PAR_NZ = par_nz;

   return Nzbin;
}
#undef __FUNC__

#define __FUNC__ "set_my_cosmological_parameters"
cosmo_lens *set_my_cosmological_parameters(error **err)
{
   int Nzbin;
   double *par_nz;
   int *Nnz;
   nofz_t *nofz;

   /* Parameters:
      Omega_m, Omega_de, w0_de, w1_de, h_100, Omega_b, Omega_nu_mass, Neff_nu_mass, normalization(sigma_8), n_spec,
      Nzbin, Nnz, nofz, par_nz (redshift distribution)
      nonlinear(linear, pd96, smith03),
      transfer(bbks, eisenhu),
      growth(heath, growth_de),
      de_param(jassal, linder, earlyDE),
      normmode(norm_s8)
   */

   Nzbin = set_redshift_distribution_single(&Nnz, &nofz, &par_nz, err);
   //Nzbin = set_redshift_distribution_tomo(&Nnz, &nofz, &par_nz, err);

   return init_parameters_lens(0.25, 0.75, -1.0, 0.0, 0.72, 0.043017, 0.0, 0.0, 0.8, 0.946,
			       Nzbin, Nnz, nofz, par_nz, smith03, eisenhu, growth_de, linder,
			       norm_s8, tomo_all, err);

   //return set_cosmological_parameters_to_default(err);
}
#undef __FUNC__

int main(int argc, char** argv)
{
   cosmo_lens* model;
   double k, ell, theta, a, pk, f, z;
   int i_bin, j_bin, Nzbin;
   error *myerr = NULL, **err;
   FILE *F;
   //datcov *dc;

   err = &myerr;

   printf("# lensingdemo (M. Kilbinger, 2006-2009)\n");

   /* Setting up cosmology */

   /*
   model = set_my_cosmological_parameters(err);
   exitOnError(*err, stderr);
   */

   F = fopen_err("cosmo_lens.par", "r", err);
   exitOnError(*err, stderr);
   read_cosmological_parameters_lens(&model, F, err);
   exitOnError(*err, stderr);
   fclose(F);

   Nzbin = model->redshift->Nzbin;

   printf("# Cosmological parameters:\n");
   dump_param_lens(model, stdout, err);
   exitOnError(*err, stderr);
   printf("a_min = %g\n", model->cosmo->a_min);


   for (i_bin=0; i_bin<Nzbin; i_bin++) {
      printf("Mean redshift (bin %d) = %.3f\n", i_bin, zmean(model->redshift, i_bin, err));
      exitOnError(*err, stderr);
   }

   /* Density power spectrum */

   printf("Calculating density power spectrum, writing file P_delta\n");
   F = fopen("P_delta", "w");
   dump_param_lens(model, F, err);
   exitOnError(*err, stderr);

   a = 1.0;
   fprintf(F, "# k[Mpc/h] Delta_NL(a=%.2f,k)\n", a);
   for (k=0.0001; k<=100.0; k*=1.05) {
      //f = k*k*k/(2.0*pi_sqr);
      f = 1.0;
      fprintf(F, "%e %e ", k, P_NL(model->cosmo, a, k, err)*f*f*f);
      fprintf(F, "\n");
      exitOnError(*err, stderr);
   }
   fclose(F);


   /* Redshift distribution */
   printf("Printing redshift distribution to file nz\n");
   F = fopen("nz", "w");
   dump_param_lens(model, F, err);
   exitOnError(*err, stderr);

   for (z=0; z<=10; z+=0.01) {
      fprintf(F, "%.3f", z);
      for (i_bin=0; i_bin<Nzbin; i_bin++) {
	 fprintf(F, " %g", prob(model->redshift, z, i_bin, err));
	 exitOnError(*err, stderr);
      }
      fprintf(F, "\n");
   }
   fclose(F);


   /* Convergence power spectrum */

   printf("Calculating convergence power spectrum, writing file P_kappa\n");
   F = fopen("P_kappa", "w");
   dump_param_lens(model, F, err);
   exitOnError(*err, stderr);

   fprintf(F, "# l P_kappa(l) for %d z-bins\n", Nzbin);
   for (ell=2.0; ell<=1.0e5; ell*=1.5) {
      f = ell*(ell+1)/twopi;
      fprintf(F, "%e", ell);
      for (i_bin=0; i_bin<Nzbin; i_bin++) {
	 for (j_bin=i_bin; j_bin<Nzbin; j_bin++) {
	    if (model->tomo==tomo_auto_only && i_bin!=j_bin) continue;
	    if (model->tomo==tomo_cross_only && i_bin==j_bin) continue;
	    pk = Pkappa(model, ell, i_bin, j_bin, err);
	    exitOnError(*err, stderr);
	    fprintf(F, " %e", pk*f);
	 }
      }
      fprintf(F, "\n");
   }
   fclose(F);


   /* Shear correlation functions */

   printf("Calculating shear correlation function, writing files xi_p, xi_m\n");
 
   F = fopen("xi_p", "w");
   dump_param_lens(model, F, err);
   exitOnError(*err, stderr);

   fprintf(F, "# theta[arcmin] xi+(theta)\n");
   for (theta=1*arcmin; theta<50*arcmin; theta*=1.1) {
      fprintf(F, "%9.3f", theta/arcmin);
      for (i_bin=0; i_bin<Nzbin; i_bin++) {
	 for (j_bin=i_bin; j_bin<Nzbin; j_bin++) {
	    if (model->tomo==tomo_auto_only && i_bin!=j_bin) continue;
	    if (model->tomo==tomo_cross_only && i_bin==j_bin) continue;
	    fprintf(F, " %e", xi(model, +1, theta, i_bin, j_bin, err));
	    exitOnError(*err, stderr);
	 }
      }
      fprintf(F, "\n");
   }
   fclose(F);

   F = fopen("xi_m", "w");
   dump_param_lens(model, F, err);
   exitOnError(*err, stderr);

   fprintf(F, "# theta[arcmin] xi-(theta)\n");
   for (theta=1*arcmin; theta<50*arcmin; theta*=1.1) {
      fprintf(F, "%9.3f", theta/arcmin);
      for (i_bin=0; i_bin<Nzbin; i_bin++) {
	 for (j_bin=i_bin; j_bin<Nzbin; j_bin++) {
	    if (model->tomo==tomo_auto_only && i_bin!=j_bin) continue;
	    if (model->tomo==tomo_cross_only && i_bin==j_bin) continue;
	    fprintf(F, " %e", xi(model, -1, theta, i_bin, j_bin, err));
	    exitOnError(*err, stderr);
	 }
      }
      fprintf(F, "\n");
   }
   fclose(F);


   /* Tophat shear variance */

   printf("Calculating tophat shear variance, writing file gammasqr\n");
   F = fopen("gammasqr", "w");
   dump_param_lens(model, F, err);
   exitOnError(*err, stderr);

   fprintf(F, "# theta[arcmin] <|gamma|^2>(theta)\n");
   for (theta=1*arcmin; theta<50*arcmin; theta*=1.1) {
      fprintf(F, "%9.3f", theta/arcmin);
      for (i_bin=0; i_bin<Nzbin; i_bin++) {
	 for (j_bin=i_bin; j_bin<Nzbin; j_bin++) {
	    if (model->tomo==tomo_auto_only && i_bin!=j_bin) continue;
	    if (model->tomo==tomo_cross_only && i_bin==j_bin) continue;
	    fprintf(F, " %e", gamma2(model, theta, i_bin, j_bin, err));
	    exitOnError(*err, stderr);
	 }
      }
      fprintf(F, "\n");
   }
   fclose(F);


   /* Aperture mass variance */

   printf("Calculating aperture mass variance (polynomial filter), writing files mapsqr, mapsqr_gauss\n");
   F = fopen("mapsqr", "w");
   dump_param_lens(model, F, err);
   exitOnError(*err, stderr);

   fprintf(F, "# theta[arcmin] <M_ap^2>(theta){polynomial}\n");
   for (theta=1*arcmin; theta<50.0*arcmin; theta*=1.1) {
      fprintf(F, "%9.3f", theta/arcmin);
      for (i_bin=0; i_bin<Nzbin; i_bin++) {
	 for (j_bin=i_bin; j_bin<Nzbin; j_bin++) {
	    if (model->tomo==tomo_auto_only && i_bin!=j_bin) continue;
	    if (model->tomo==tomo_cross_only && i_bin==j_bin) continue;
	    fprintf(F, " %e", map2_poly(model, theta, i_bin, j_bin, err));
	    exitOnError(*err, stderr);
	 }
      }
      fprintf(F, "\n");
   }
   fclose(F);

   F = fopen("mapsqr_gauss", "w");
   dump_param_lens(model, F, err);
   exitOnError(*err, stderr);

   fprintf(F, "# theta[arcmin] <M_ap^2>(theta){polynomial}\n");
   for (theta=1*arcmin; theta<50.0*arcmin; theta*=1.1) {
      fprintf(F, "%9.3f", theta/arcmin);
      for (i_bin=0; i_bin<Nzbin; i_bin++) {
	 for (j_bin=i_bin; j_bin<Nzbin; j_bin++) {
	    if (model->tomo==tomo_auto_only && i_bin!=j_bin) continue;
	    if (model->tomo==tomo_cross_only && i_bin==j_bin) continue;
	    fprintf(F, " %e", map2_gauss(model, theta, i_bin, j_bin, err));
	    exitOnError(*err, stderr);
	 }
      }
      fprintf(F, "\n");
   }
   fclose(F);


   /*
   //dc = init_data_cov_tomo("Map.ascii", "mapcov_cosmos_w0.dat", map2poly, err);
   dc = init_data_cov_tomo("map2.cut_1_2_3_4_newformat", "CovMap51_0.95+B.col.cut_1_2_3_4.symmetric",
			   map2poly, err);
   //dc = init_data_cov_tomo("test.dat", "test.cov", map2poly, err);
   exitOnError(*err, stderr);

   f = chi2_lensing(model, dc, err);
   exitOnError(*err, stderr);
   printf("chi^2 = %g\n", f);

   del_data_cov(&dc);
   */

   printf("Done\n");

   return 0;
}
