/* ============================================================ *
 * config.h							*
 * Martin Kilbinger 2007					*
 * ============================================================ */

#ifndef __CONFIG_H
#define __CONFIG_H

#include <stdio.h>
#include <string.h>

#include "errorlist.h"
#include "io.h"


#define conf_base  -400
#define conf_undef -1 + conf_base
#define conf_io    -2 + conf_base
#define conf_eof   -3 + conf_base


typedef enum {c_i, c_d, c_s} config_element_t;

typedef struct {
  int i;
  double d;
  char s[512];
} config_element;


/* ============================================================ *
 * The following macros read entries from a configuration file. *
 * In the calling function the variables c (of type             *
 * config_element) and s (char[]) have to be defined.           *
 * ============================================================ */


/* Reads a config entry of the form "key    value" */
#define CONFIG_READ(config, element, type, F, c, err)			\
  (config)->element = read_element(F, #element, c, c_##type, 1, err).type; \
  forwardError(*err, __LINE__,)


/* Reads a config entry of type 'string' of the form "key   string " */
#define CONFIG_READ_S(config, element, s, F, c, err)			\
   strcpy((config)->element, read_element(F, #element, c, c_s, 1, err).s); \
   forwardError(*err, __LINE__,)


/* Reads an array of config entries of the form "key   value1 value2 value3 ... valuen" */
#define CONFIG_READ_ARR(config, element, type, i, n, s, F, c, err)	\
   STRING_READ(F, #element, err); \
   for (i=0; i<n; i++) { \
      (config)->element[i] = read_element(F, "", c, c_##type, i==n-1, err).type; \
      forwardError(*err, __LINE__,); \
   }


/* Reads an array of config entries of type 'string' of the form "key   string1 string2 string3 ... stringn" */
#define CONFIG_READ_S_ARR(config, element, s, i, n, F, c, err)		\
   STRING_READ(F, #element, err);					\
   for (i=0; i<n; i++) { \
      strcpy((config)->element[i], read_element(F, "", c, c_s, i==n-1, err).s); \
      forwardError(*err, __LINE__,); \
   }



/* Assigns a value to config->element according to the string selement */
#define STRING2ENUM(config, element, selement, type, stype, N, err) \
   (config)->element = -1; \
    for (j=0; j<N; j++) { \
       strcpy(line, stype(j)); \
       if (strcmp((config)->selement, stype(j))==0) (config)->element = (type)j; \
    } \
  testErrorRetVA((config)->element==-1, conf_undef, "Parametrization '%s' not found in type %s, cannot assign value of type %s", *err, __LINE__,, (config)->selement, #stype, #type)

/*
#define STRING2ENUM_SHORT(config, element, err)			\
  STRING2ENUM(config, element, s#element, #element_t s#element_t, N#element_t, err)
*/


config_element read_element(FILE *F, char *key, config_element c,
			    config_element_t t, int all, error **err);


/* =============================================================================== */


/* This two macros should be used called internally (by the CONFIG_READ... macros) */
#define ENTRY_READ(F, key, type, err) \
    (c = read_element(F, key, c, c_##type, 0, err), c.type)

#define STRING_READ(F, str, err) \
    strcpy(s, ENTRY_READ(F, "", s, err)); \
    testErrorRetVA(strcmp(s, str)!=0, conf_undef, \
    "String \"%s\" expected but \"%s\" found instead in config file", *err, __LINE__,, str, s);



#endif

