/*
 *  errorlist.h
 *  likeli
 *
 *  Created by Karim Benabed on 06/02/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __ERRORLIST_H
#define __ERRORLIST_H


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>


#define TXT_SZ 4192
#define WHR_SZ 2048
#define TOT_SZ TXT_SZ+WHR_SZ

typedef struct _err {
	char errWhere[WHR_SZ];
	char errText[TXT_SZ];
	int errValue;
	struct _err* next;
} error;
typedef error sm2_error;

#define forwardErr -123456789
#define undefErr -987654321
#define noErr 0

#define el_alloc  -42

error* newError(int errV, char* where, char* text, error* linkTo, error* addErr);
error* newErrorVA(int errV, char* where, char* text, error* linkTo, error* addErr,...);

void stringError(char* str, error *err);
void printError(FILE* flog,error* err);
int getErrorValue(error* err);

void purgeError(error **err); 

error* unpickleError(void* buf, int len);
int pickleError(error* err, void** buf);


#define _DEBUGHERE_(extra,...) fprintf(stderr,__FUNC__":("__FILE__":%d) "extra"\n",__LINE__,__VA_ARGS__);

#define isError(err)                         (err != NULL)

#define someErrorVA(errV,txt,prev,next,li,...) newErrorVA(errV,__FUNC__"("__FILE__":"#li")",txt,prev,next,__VA_ARGS__)
#define topErrorVA(errV,txt,next,li,...)       someErrorVA(errV,txt,NULL,next,li,__VA_ARGS__)
#define oneErrorVA(errV,txt,li,...)            someErrorVA(errV,txt,NULL,NULL,li,__VA_ARGS__)
#define addErrorVA(errV,txt,prev,li,...)       someErrorVA(errV,txt,prev,NULL,li,__VA_ARGS__)

#define someError(errV,txt,prev,next,li)       someErrorVA(errV,txt,prev,next,li,"")
#define topError(errV,txt,next,li)             topErrorVA(errV,txt,next,li,"")
#define oneError(errV,txt,li)                  oneErrorVA(errV,txt,li,"")
#define addError(errV,txt,prev,li)             addErrorVA(errV,txt,prev,li,"")


#define forwardError(err,line,ret) { \
  if (isError(err)) { \
		err=topError(forwardErr,"",err,line); \
		return ret; \
  } \
}

#define forwardErrorNoReturn(err,line) { \
  if (isError(err)) { \
                err=topError(forwardErr,"",err,line); \
  } \
}

#define testErrorRetVA(test,error_type,message,err,line,return_value,...) {\
  if (test) {	\
	  err=addErrorVA(error_type,message,err,line,__VA_ARGS__); \
	  return return_value; \
  } \
}
#define testErrorRet(test,error_type,message,err,line,return_value) testErrorRetVA(test,error_type,message,err,line,return_value,"")

#define testErrorVA(test,error_type,message,err,line,...) { \
  if (test) {	\
    err=addErrorVA(error_type,message,err,line,__VA_ARGS__); \
  } \
}
#define testError(test,error_type,message,err,line) testErrorVA(test,error_type,message,err,line,"")

#define testErrorExitVA(test,error_type,message,err,line,...) { \
  if (test) {	\
    err=addErrorVA(error_type,message,err,line,__VA_ARGS__); \
    printError(stderr,err);\
    exit(getErrorValue(err)); \
  }\
}
#define testErrorExit(test,error_type,message,err,line) testErrorExitVA(test,error_type,message,err,line,"")

#define exitOnError(err,F) {\
  if(isError(err)) {\
	printError(F,err);\
	exit(getErrorValue(err)); \
  }\
}

#define quitOnError(err,line,F) {\
forwardErrorNoReturn(err,line);\
if(isError(err)) {\
printError(F,err);\
exit(getErrorValue(err)); \
}\
}

#define err_base      -100
#define err_io        -1 + err_base
#define err_allocate  -2 + err_base


#endif
