#include <assert.h>
#include <stdio.h>
#include <string.h>
#include "config.h"


#define __FUNC__ "read_element"
config_element read_element(FILE *F, char *key, config_element c,
			    config_element_t t, int all, error **err)
{
   int i, n, commentline;
   double d;
   char s[1024], formatkey[1024];
   char *line=NULL;
   size_t nread;

   c.i = 0;
   c.d = 0.0;
   strcpy(c.s, "");
   strcpy(formatkey, key);

   line = malloc(1024*sizeof(char));

   /* Reads from file, discards comment lines (starting with '#') and empty lines */
   if (all==0) {

      /* Read only one string from file */
      do {
	 nread = fscanf(F, "%s", line);
	 if (line[0]=='#' || line[0]=='\n') {
	    fgets(line, 1024, F);
	    commentline = 1;
	 } else {
	    commentline = 0;
	 }
      } while (commentline==1);

   } else {

      /* Read whole line and discard rest after value */
      do {
	 fgets(line, 1024, F);
	 //fprintf(stderr, "(%s) %d\n", line, 1);
      } while (line[0]=='#' || line[0]=='\n');

   }
   //fprintf(stderr, "all=%d,line=(%s)\n", all, line);

   switch (t) {
      case c_i :
	 strcat(formatkey, " %d ");
	 n = sscanf(line, formatkey, &i);
	 c.i = i;
	 break;

      case c_d :
	 strcat(formatkey, " %lf ");
	 n = sscanf(line, formatkey, &d);
	 c.d = d;
	 break;

      case c_s :
	 strcat(formatkey, " %s ");
	 n = sscanf(line, formatkey, s);
	 strcpy(c.s, s);
	 //fprintf(stderr, "read %d entry (%s)\n", n, s);
	 break;

      default : 
	 *err = addError(conf_undef, "config_element type unknown", *err,
			 __LINE__);
	 c.i = -1; 
	 c.d = -1.0; 
	 strcpy(c.s, "");
         free(line);
	 return c;
   }

   if (n==0 || n==EOF) {
      if (n==0) {
	 chomp(line);
         *err = addErrorVA(conf_io, "String %s expected but not found in config file, line '%s'",
			   *err, __LINE__, key, line);
      } else {
	 *err = addErrorVA(conf_io, "Unexpected eof in config file, line '%s'", *err, __LINE__, line);
      }

      free(line);
      c.i = -1; 
      c.d = -1.0; 
      strcpy(c.s, "");
      return c;
   }

   return c;
}
#undef __FUNC__

