/*
 *  mvdens.c
 *  likely
 *
 *  Created by Karim Benabed on 10/03/08.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#ifdef __PLANCK__
#include "HL2_likely/tools/mvdens.h"
#else
#include "mvdens.h"
#endif

#define __FUNC__ "mvdens_alloc"
mvdens * mvdens_alloc(size_t size, error **err) {
  mvdens *g;
  size_t ndim;

  ndim = size;
  g = (mvdens *) malloc_err(sizeof(mvdens),err);
  forwardError(*err,__LINE__,NULL);
  g->buf = calloc_err(ndim*(ndim+1), sizeof(double),err);
  forwardError(*err,__LINE__,NULL);
  
  g = mvdens_init(g,size,g->buf,((double*) g->buf)+ndim,err);
  forwardError(*err,__LINE__,NULL);
  g->own_buf=1;

  return g;
}

#undef __FUNC__
#define __FUNC__ "mvdens_init"
mvdens * mvdens_init(mvdens *g, size_t size,double* mn, double *std, error **err) {
  g->ndim=size;
  
  g->mean = mn;
  
  /* This is used to store the square root (Cholesky) of the cov.   */
  g->std = std;
  
  gsl_set_error_handler_off();
  g->mean_view_container = gsl_vector_view_array(g->mean,g->ndim);
  g->mean_view = &(g->mean_view_container.vector);
  g->std_view_container = gsl_matrix_view_array(g->std,g->ndim,g->ndim); 
  g->std_view   = &(g->std_view_container.matrix);
  g->band_limit = size;
  g->chol       = 0;
  g->df         = -1;
  g->own_buf    = 0;
  g->detL       = -1.0;

  return g;
}
#undef __FUNC__

#define __FUNC__ "mvdens_from_meanvar"
void mvdens_from_meanvar(mvdens *m, const double *pmean, const double *pvar, double correct) {
  int j, k;
  for (j=0; j<m->ndim; j++) {
    for (k=0; k<m->ndim; k++) {
      m->std[j*m->ndim+k] = pvar[j*m->ndim+k]*correct;
    }
    if (pmean!=NULL) 
      m->mean[j] = pmean[j];
  }
}
#undef __FUNC__

void mvdens_empty(mvdens *g) {
  if (g->own_buf==1) {
    free(g->buf);
  }
}

void mvdens_free(mvdens ** g) {
  mvdens_empty(*g);
  free(*g);
  *g = NULL;
}


#define __FUNC__ "mvdens_print"
void mvdens_print(FILE* where, mvdens* what) {
  FILE *rhere;
  size_t j,k,ndim;
  
  rhere=where;
  if (where==NULL) {
    rhere=stdout;
  }
  ndim=what->ndim;
  if(what->df!=-1)
    fprintf(rhere, "df = %d\n", what->df);
  if(what->chol==1)
    fprintf(rhere, "Beware! Cholesky decomposed std, df=%d\n", what->df);
  for (j=0;j<ndim;j++) {
    fprintf(rhere,"% .3g | ",what->mean[j]);
    for (k=0;k<j;k++) 
      fprintf(rhere,"% .3g ",what->std[j*ndim+k]);
    fprintf(rhere,"% .3g\n",what->std[j*ndim+j]);
    //fprintf(rhere,"\n");
  }
  return;
}
#undef __FUNC__

void mvdens_dump(FILE* where, mvdens* what) {
  FILE *rhere;
  size_t j,k,ndim;
  
  rhere=where;
  if (where==NULL) {
    rhere=stdout;
  }
  ndim=what->ndim;
  fprintf(rhere, "%zd %d %zd %d\n", ndim, what->df, what->band_limit, what->chol);
  for (j=0;j<ndim;j++) {
    fprintf(rhere,"%.5g ",what->mean[j]);
  }
  fprintf(rhere,"\n");
  for (j=0;j<ndim;j++) {
    for (k=0;k<ndim;k++) 
      fprintf(rhere,"%.5g ",what->std[j*ndim+k]);
    fprintf(rhere,"\n");
  }
  return;
}

#define __FUNC__ "mvdens_chdump"
void mvdens_chdump(const char *name, mvdens* what, error **err)
{
   FILE *F;

   F = fopen_err(name, "w", err);
   forwardError(*err, __LINE__,);
   mvdens_dump(F, what);
   fclose(F);
}
#undef __FUNC__

#define __FUNC__ "mvdens_dwnp"
mvdens* mvdens_dwnp(FILE* where, error **err) {
  mvdens *what;
  FILE *rhere;
  size_t j,k,ndim,df,bdl,chol;
  
  rhere=where;
  if (where==NULL) {
    rhere=stdin;
  }
  testErrorRet(fscanf(rhere,"%zu %zu %zu %zu\n",&ndim,&df,&bdl, &chol)!=4,
	       mv_io, "Error while reading mvdens file: Four integers \n"
	       "(ndim, dof, bandlimit, cholesky flag) expected on first line",
	       *err, __LINE__, NULL);
  
  what=mvdens_alloc(ndim,err);
  forwardError(*err,__LINE__,NULL);
  
  for (j=0; j<ndim; j++) {
    testErrorRetVA(fscanf(rhere, "%lg", what->mean+j)!=1, mv_io,
		   "Cannot read mvdens (mean element #%d) double expected",
		   *err, __LINE__, NULL, j);
  }
  for (j=0; j<ndim; j++) {
    for (k=0; k<ndim; k++) {
      testErrorRetVA(fscanf(rhere, "%lg", what->std+j*ndim+k)!=1, mv_io,
		     "Cannot read mvdens (covariance element #(%d,%d)) double expected",
		     *err, __LINE__, NULL, j, k);
     }
  }
  
  what->band_limit = bdl;
  what->df         = df;
  what->chol       = chol;
  if (what->chol==1) {
     what->detL = determinant(what->std, what->ndim);
  }

  return what;
}
#undef __FUNC__

#define __FUNC__ "mvdens_read_and_chol"
mvdens *mvdens_read_and_chol(FILE *where, error **err)
{
   mvdens *what;

   what = mvdens_dwnp(where, err);
   forwardError(*err, __LINE__, NULL);
   if (what->chol==0) {
      mvdens_cholesky_decomp(what, err);
      forwardError(*err, __LINE__, NULL);
   }

   //fprintf(stderr, "mvdens_read_and_chol: det = %g\n", what->detL);

   return what;
}
#undef __FUNC__

/* L has to be a trianglular matrix (e.g. after Cholesky-decomposition) */
double determinant(const double *L, int ndim)
{
   int i;
   double det;

   for (i=0,det=1.0; i<ndim*ndim; i+=(ndim+1))
     det *= L[i];

   return det;
}

#define __FUNC__ "mvdens_cholesky_decomp"
void mvdens_cholesky_decomp(mvdens* self, error **err) {
  gsl_set_error_handler_off();
  if (self->chol==1)
    return;

#ifdef __MVDENS_PARANOID_DEBUG__
  double * _std;
  _std=malloc_err(self->ndim*self->ndim*sizeof(double),err);
  forwardError(*err,__LINE__,);
  memcpy(_std,self->std,sizeof(double)*self->ndim*self->ndim);
#endif  

  if (gsl_linalg_cholesky_decomp(self->std_view) ==  GSL_EDOM) {

#ifdef __MVDENS_PARANOID_DEBUG__
    memcpy(self->std, _std, sizeof(double)*self->ndim*self->ndim);
    free(_std);
#endif

    *err = addError(mv_cholesky,"Cholesky decomposition failed",*err,__LINE__);
    return;
  }

  self->detL = determinant(self->std, self->ndim);

#ifdef __MVDENS_PARANOID_DEBUG__
  free(_std);
#endif

  self->chol=1;
}  
#undef __FUNC__

#define __FUNC__ "mvdens_ran"
double* mvdens_ran(double* dest, mvdens * g, gsl_rng * r,error **err) {
  size_t i;
  double *res;
  gsl_vector * res_view;
  gsl_vector_view res_view_container;
  double val,corr,u;
  
  mvdens_cholesky_decomp(g,err);
  forwardError(*err,__LINE__,NULL);
  
  MALLOC_IF_NEEDED(res,dest,sizeof(double)*g->ndim,err);
  forwardError(*err,__LINE__,NULL);

  gsl_set_error_handler_off();
  /* Generate a N(O,I) distributed vector               */
  for (i = 0; i < g->ndim; i++) {
    val=gsl_ran_gaussian(r, 1.0);
    res[i]=val;
  }
  
  /* Make it N(O,Sigma)                         */
  res_view_container=gsl_vector_view_array(res,g->ndim);
  res_view=&(res_view_container.vector);
  gsl_blas_dtrmv(CblasLower, CblasNoTrans, CblasNonUnit, g->std_view, res_view);

  corr=1;
  if(g->df!=-1) {
    // make it student-t
    u = gsl_ran_chisq(r, g->df);
    corr=sqrt(g->df/u);
  }
 
  // add mean and correct if student-t
  for(i=0;i<g->ndim;i++) {
    res[i]*=corr;
    res[i]+=g->mean[i];
  }
  return res;
}
#undef __FUNC__

#define __FUNC__ "scalar_product"
double scalar_product(mvdens *g, const double *x, error **err) {
  gsl_vector * x_tmp_view;
  gsl_vector_view x_tmp_view_container;
  double *x_tmp;
  double logl;
  size_t i, ndim;
  
  gsl_set_error_handler_off();
  
  mvdens_cholesky_decomp(g,err);
  forwardError(*err,__LINE__,-1);
  
  ndim = g->ndim;
  /* Copy vector */
  x_tmp = malloc_err(sizeof(double)*ndim,err);
  forwardError(*err,__LINE__,0);
  x_tmp_view_container = gsl_vector_view_array(x_tmp, ndim);
  x_tmp_view           = &(x_tmp_view_container.vector);
  
  memcpy((void*)x_tmp,(void*)x,(size_t)sizeof(double)*ndim);
  
  /* Subtract mean */
  gsl_vector_sub(x_tmp_view, g->mean_view);

  /* Compute Sigma^{-1/2}(x-mu) (by substitution) */
  gsl_blas_dtrsv(CblasLower, CblasNoTrans, CblasNonUnit, g->std_view, x_tmp_view);

  /* Compute squared norm */
  for (i=0,logl=0.0; i<ndim; i++) {
    logl += x_tmp[i]*x_tmp[i];
  }
  
  free(x_tmp);
  
  return logl;
}
#undef __FUNC__

/* ============================================================ *
 * Returns the negative log-likelihood for a mv Gaussian pdf.   *
 * ============================================================ */
#define __FUNC__ "mvdens_log_pdf"
double mvdens_log_pdf(mvdens *g, const double *x, error **err) {
  double log_g, logdet;
  size_t ndim, n, p;
  double gamma, norm, r;

  gsl_set_error_handler_off();
  
  mvdens_cholesky_decomp(g, err);
  forwardError(*err,__LINE__,-1);
  
  ndim   = g->ndim;
  logdet = log(g->detL);
  
  /* log_g contains the scalar product xt . var . x */
  log_g = scalar_product(g, x, err);
  
  forwardError(*err,__LINE__,-1);  
  
  /* Student-t distribution with df degrees of freedom */
  if (g->df!=-1) {
    n = g->df;
    p = g->ndim;
    
    gamma = gsl_sf_lngamma(0.5*(n+p));
    norm  = gsl_sf_lngamma(0.5*n) + 0.5*p*log(n*PI);
    r     = 0.5*(n+p)*log(1.0+(1.0/n)*log_g);
    return gamma - (norm + logdet + r);
  } else {
     /* logdet = log|Sigma| = 0.5*log|Covariance| ! */
     r = -0.5*(ndim*ln2pi + log_g) - logdet;
     return r;
  }
}
#undef __FUNC__

#define __FUNC__ "mvdens_log_pdf_void"
double mvdens_log_pdf_void(const void *g, const double *x, error **err) {
   double res;
   res = mvdens_log_pdf((mvdens*)g, x, err);
   forwardError(*err, __LINE__, 0.0);
   return res;
}
#undef __FUNC__

#define __FUNC__ "mix_mvdens_alloc"
mix_mvdens *mix_mvdens_alloc(size_t ncomp, size_t size,error **err) {
  mix_mvdens * m; 
  size_t i,dcomp,std0,ndim;
  void * data;
  
  m = (mix_mvdens *) malloc_err(sizeof(mix_mvdens),err);
  forwardError(*err,__LINE__,NULL);
  
  m->ncomp = ncomp;
  m->ndim  = size;
  m->wght  = calloc_err(ncomp,sizeof(double),err);
  forwardError(*err,__LINE__,NULL);

  gsl_set_error_handler_off();
  
  m->wght_view_container=gsl_vector_view_array(m->wght,ncomp);
  m->wght_view=&(m->wght_view_container.vector);
  
  m->cwght = calloc_err(ncomp,sizeof(double),err);
  forwardError(*err,__LINE__,NULL);
  m->cwght_view_container=gsl_vector_view_array(m->cwght,ncomp);
  m->cwght_view=&(m->cwght_view_container.vector);
  
  
  
  m->comp = (mvdens **) malloc_err(ncomp * sizeof(mvdens *),err);
  forwardError(*err,__LINE__,NULL);
  
  ndim = m->ndim;
  m->buf_comp = malloc_err(ncomp*(sizeof(mvdens)+ndim*(ndim+1)*sizeof(double)),err);
  forwardError(*err,__LINE__,NULL);
  memset(((char*)m->buf_comp)+ncomp*sizeof(mvdens),0,ncomp*(ndim*(ndim+1)*sizeof(double)));
  data  = ((char*)m->buf_comp)+ncomp * sizeof(mvdens);
  dcomp = ndim*(ndim+1)*sizeof(double);
  std0  = ndim*sizeof(double);
  
  for (i=0; i<ncomp; i++) {
    m->comp[i] = mvdens_init((mvdens*)((char*)(m->buf_comp)+i*sizeof(mvdens)), size,
			     (double*)((char*)data+dcomp*i), (double*)((char*)data+dcomp*i+std0), err);
    forwardError(*err,__LINE__,NULL);
  }
  
  m->init_cwght = 0;
  
  return m;
  
}
#undef __FUNC__

void mix_mvdens_free(mix_mvdens **m) {
  size_t i;
  
  for (i = 0; i < (*m)->ncomp; i++) {
    mvdens_empty((*m)->comp[i]);
  }
  free((*m)->comp);
  free((*m)->buf_comp);
  free((*m)->wght);
  free((*m)->cwght);
  free(*m);
  *m=NULL;
}

#define __FUNC__ "mix_mvdens_print"
void mix_mvdens_print(FILE* where,mix_mvdens* what) {
  FILE *rhere;
  int i, ncomp;
  
  rhere=where;
  if (where==NULL) {
    rhere=stdout;
  }
  
  ncomp=what->ncomp;
  
  for (i=0; i<ncomp; i++) {
    fprintf(rhere, "*************************************\n");
    fprintf(rhere, "Weight of component #%d is %g\n", i, what->wght[i]);
    if (what->wght[i]!=0) {
      mvdens_print(rhere, what->comp[i]);
    } 
  }
}
#undef __FUNC__

#define __FUNC__ "mix_mvdens_dump"
void mix_mvdens_dump(FILE* where,mix_mvdens* what) {
  FILE *rhere;
  int i,ncomp;

  rhere=where;
  if (where==NULL) {
    rhere=stdout;
  }

  ncomp=what->ncomp;
  fprintf(rhere, "%d %d\n", (int)what->ncomp, (int)what->ndim);
  for (i=0;i<ncomp;i++) {
    fprintf(rhere,"%g\n",what->wght[i]);
    mvdens_dump(rhere,what->comp[i]);
  }
}
#undef __FUNC__

#define __FUNC__ "mix_mvdens_dwnp"
mix_mvdens* mix_mvdens_dwnp(FILE* where, error **err) {
  mix_mvdens *what;
  FILE *rhere;
  size_t i,j,k,ndim,ncomp;
  size_t df,bdl,chol;
  mvdens *w;
  
  rhere=where;
  if (where==NULL) {
    rhere=stdin;
  }
  testErrorRet(fscanf(rhere,"%zu %zu\n",&ncomp,&ndim)!=2,mv_io,
	       "Cannot read mix_mvdens first line (two integers expected)",*err,__LINE__,NULL);

  //fprintf(stderr,"mix_mvdens_dwnp -> %ld %ld\n",ncomp,ndim);
  what=mix_mvdens_alloc(ncomp,ndim,err);
  forwardError(*err,__LINE__,NULL);
  
  for(i=0;i<ncomp;i++) {
    testErrorRetVA(fscanf(rhere,"%lg\n",&(what->wght[i]))!=1,mv_io,"Cannot read mix_mvdens (comp %d weight)",*err,__LINE__,NULL,i);
    testErrorRetVA(fscanf(rhere,"%zu %zu %zu %zu\n",&ndim,&df,&bdl, &chol)!=4,mv_io,
		   "Cannot read mix_mvdens (comp %d first line)",*err,__LINE__,NULL,i);
    //fprintf(stderr,"mix_mvdens_dwnp(%d) -> %ld %ld %ld %ld\n",i,ndim,df,bdl, chol);
    testErrorRetVA(ndim!=what->ndim,mv_io,
		   "Cannot read mix_mvdens, comp %d ndim invalid (expected %d got %d)",*err,__LINE__,NULL,i,what->ndim,ndim);
    
    w=what->comp[i];
    
    for (j=0; j<ndim; j++) {
      testErrorRetVA(fscanf(rhere, "%lg", w->mean+j)!=1,mv_io,"Cannot read mix_mvdens comp %d mean %d",*err,__LINE__,NULL,i,j);
      //fprintf(stderr,"mix_mvdens_dwnp(%d) %d->%g\n",i,j,w->mean[j]);
    }
    //fprintf(stderr,"mix_mvdens_dwnp(%d) std\n",i);
    for (j=0; j<ndim; j++) {
      for (k=0; k<ndim; k++) {
        testErrorRetVA(fscanf(rhere, "%lg", w->std+j*ndim+k)!=1,mv_io,
		       "Cannot read mix_mvdens comp %d var %d,%d",*err,__LINE__,NULL,i,j,k)      //fprintf(stderr,"%g ",w->std+j*ndim+k);
      }
      //fprintf(stderr,"\n");
    }
    
    w->band_limit=bdl;
    w->df=df;
    w->chol=chol;
    if (w->chol==1) w->detL = determinant(w->std, w->ndim);
  }

  return what;
}
#undef __FUNC__

#define __FUNC__ "mix_mvdens_size"
size_t mix_mvdens_size(size_t ncomp,size_t ndim) {
  size_t rsz;

  rsz  = 0;
  rsz += sizeof(size_t)*2;                     /* ncomp, ndim */
  rsz += ncomp*sizeof(double);                 /* wght */
  rsz += ncomp*sizeof(int);                    /* df */
  rsz += ncomp*sizeof(size_t);                 /* band_limit */
  rsz += ncomp*ndim*(1+ndim)*sizeof(double);   /* mean, std */

  return rsz;  
}
#undef __FUNC__

#define add_size(me,type,n) me=((char*) me)+(sizeof(type)*n)
#define store(me,type,what) (*(type*)me=what); add_size(me,type,1);
#define unstore(where,me,type) where=*(type*)me; add_size(me,type,1);


#define __FUNC__ "serialize_mix_mvdens"
void* serialize_mix_mvdens(mix_mvdens* self, size_t *sz,error** err)
{
  void *serialized, *crt;
  size_t rsz,i;
  
  //fprintf(stderr,"seri : 1\n");
  
  rsz = mix_mvdens_size(self->ncomp,self->ndim);
  serialized = malloc_err(rsz,err);
  forwardError(*err,__LINE__,NULL);
  
  //fprintf(stderr,"seri : 2\n");
  
  crt=serialized;
  //fprintf(stderr,"%p %p\n",crt,serialized);
  store(crt,size_t,self->ncomp);

  //fprintf(stderr,"%p %p\n",crt,serialized);
  store(crt,size_t,self->ndim);

  //fprintf(stderr,"%p %p\n",crt,serialized);
  memcpy(crt,self->wght,sizeof(double)*self->ncomp);
  add_size(crt,double,self->ncomp);
  //fprintf(stderr,"%p %p\n",crt,serialized);
  for(i=0;i<self->ncomp;i++) {
    //fprintf(stderr,"seri : 2 %d a\n",i);

    store(crt,int,self->comp[i]->df);
    //fprintf(stderr,"%p %p\n",crt,serialized);
    //fprintf(stderr,"seri : 2 %d b\n",i);

    store(crt,size_t,self->comp[i]->band_limit);
    //fprintf(stderr,"%p %p\n",crt,serialized);
    //fprintf(stderr,"seri : 2 %d c\n",i);

    mvdens_cholesky_decomp(self->comp[i],err);
    forwardError(*err,__LINE__,NULL);
    //fprintf(stderr,"seri : 2 %d d\n",i);

    memcpy(crt,self->comp[i]->mean,self->ndim*sizeof(double));
    add_size(crt,double,self->ndim);
    //fprintf(stderr,"%p %p\n",crt,serialized);
    //fprintf(stderr,"seri : 2 %d e\n",i);

    memcpy(crt,self->comp[i]->std,self->ndim*self->ndim*sizeof(double));
    add_size(crt,double,self->ndim*self->ndim);
    //fprintf(stderr,"%p %p\n",crt,serialized);
    //fprintf(stderr,"seri : 2 %d e\n",i);

  }
  //fprintf(stderr,"%p %p\n",crt,serialized);
  testErrorRetVA((char*)crt-(char*)serialized!=rsz, mv_serialize,
		 "Bad size for serialization (expected %d, got %d)",
		 *err, __LINE__, NULL, rsz, (char*)crt-(char*)serialized);
  //fprintf(stderr,"seri : 3\n");
  *sz=rsz;
  return serialized;
}
#undef __FUNC__

#define __FUNC__ "deserialize_mix_mvdens"
mix_mvdens* deserialize_mix_mvdens(void* serialized, size_t sz, error **err) {
  size_t ndim,ncomp,rsz,i;
  void *crt;
  mix_mvdens* m;
  
  //fprintf(stderr,"deseri : 1\n");
  
  crt=serialized;
  //fprintf(stderr,"%p %p\n",crt,serialized);
  
  unstore(ncomp,crt,size_t);
  //fprintf(stderr,"%p %p\n",crt,serialized);
  unstore(ndim,crt,size_t);
  //fprintf(stderr,"%p %p\n",crt,serialized);
  rsz = mix_mvdens_size(ncomp,ndim);
  
  testErrorRetVA(rsz!=sz,mv_serialize,"Cannot deserialize, bad size (Expected %d got %d)",*err,__LINE__,NULL,sz,rsz);

  //fprintf(stderr,"deseri : 2\n");
  m = mix_mvdens_alloc(ncomp,ndim,err);
  forwardError(*err,__LINE__,NULL);
  
  memcpy(m->wght,crt,sizeof(double)*ncomp);
  add_size(crt, double,ncomp);
  //fprintf(stderr,"%p %p\n",crt,serialized);
  
  for(i=0;i<ncomp;i++) {
    //fprintf(stderr,"deseri : 3 %d\n",i);

    unstore(m->comp[i]->df,crt,int);
    //fprintf(stderr,"%p %p\n",crt,serialized);

    unstore(m->comp[i]->band_limit,crt,size_t);
    //fprintf(stderr,"%p %p\n",crt,serialized);

    m->comp[i]->chol = 0;
    memcpy(m->comp[i]->mean,crt,sizeof(double)*ndim);
    add_size(crt,double,m->ndim);
    //fprintf(stderr,"%p %p\n",crt,serialized);

    memcpy(m->comp[i]->std,crt,sizeof(double)*ndim*ndim);
    add_size(crt,double,m->ndim*m->ndim);
    //fprintf(stderr,"%p %p\n",crt,serialized);

    m->comp[i]->chol = 1;
    /* comp was Cholesky-decomposed in serialize, therefore we can call determinant */
    m->comp[i]->detL = determinant(m->comp[i]->std, ndim);
  }
  //fprintf(stderr,"deseri : 4\n");
  //fprintf(stderr,"%p %p\n",crt,serialized);
  
  testErrorRetVA((char*)crt-(char*)serialized!=rsz, mv_serialize,
		 "Cannot deserialize, bad size (Expected %d, got %d)",
		 *err, __LINE__, NULL, rsz, (char*)crt-(char*)serialized);
    
  //fprintf(stderr,"deseri : 5\n");

  return m;
}
#undef __FUNC__ 

#define __FUNC__ "mix_mvdens_cholesky_decomp"
void mix_mvdens_cholesky_decomp(mix_mvdens* self, error **err) {
  size_t i;
  for (i=0;i<self->ncomp;i++) {
    mvdens_cholesky_decomp(self->comp[i],err);
     forwardError(*err,__LINE__,);
  }
}
#undef __FUNC__

#define __FUNC__ "mix_mvdens_ran"
double* mix_mvdens_ran(double* dest,size_t *index, mix_mvdens *m, gsl_rng * r,error **err) {
  /* Draw random realization according to mixture */
  size_t i;
  double * x;
  gsl_set_error_handler_off();
  
  if (m->init_cwght == 0) { /* Initialize cumulative weights */
    
    double sum_wght=0.0;
    
    for (i=0;i<m->ncomp;i++) 
      sum_wght += m->wght[i];
    if (fabs(sum_wght-1.0) > 1e-6) {
      testErrorRet(sum_wght==0,mv_negWeight,"All weights to zero !! cannot sample",*err,__LINE__,NULL);
      gsl_vector_scale(m->wght_view,1.0/sum_wght);
    }
    m->cwght[0]=m->wght[0];
    for (i=1;i<m->ncomp;i++) {
      m->cwght[i]=m->cwght[i-1]+m->wght[i];
    }
    m->init_cwght=1;
  }
  
  /* Draw index according to weights */
  *index = ranindx(m->cwght,m->ncomp,r,err);
  //fprintf(stderr,"%d -> ",*index);
  forwardError(*err,__LINE__,NULL);
  /* Draw gaussian random variable of corresponding law,
   the last index corresponding to the initial proposal ? */
  x = mvdens_ran(dest, m->comp[*index], r, err);
  for(i=0;i<m->ndim;i++)
    //fprintf(stderr,"%g ",x[i]);
  forwardError(*err,__LINE__,NULL);
  //fprintf(stderr,"\n");
  return x;
  
}
#undef __FUNC__

#define __FUNC__ "ranindx"
size_t ranindx(double *cw, size_t nE, gsl_rng * r,error ** err) {
  
  /* Draw index according to probability weights */
  /* Uses cumulative weight vector cw */
  
  size_t index=0;
  double z;
  gsl_set_error_handler_off();
  
  z = gsl_ran_flat(r,0.0,1.0); /* Weights are assumed normalized */
  while (cw[index]< z && index < nE-1) {
    index++;
  }
  //  printf("index is %d \n",index);
  return index;
  
}
#undef __FUNC__

#define __FUNC__ "mix_mvdens_log_pdf"
double mix_mvdens_log_pdf( const mix_mvdens *m, double *x,error **err) {
  /* Log pdf of MOG */
  
  double *logdens;
  double logdens_max_loc, logdens_tmp, lpos;
  int i,iloc;
  int ncomp = m->ncomp;
  
  //fprintf(stderr,"--->>> %d\n",ncomp);
  logdens = malloc_err(ncomp*sizeof(double),err);
  forwardError(*err,__LINE__,0);
  
  iloc=0;
  while (m->wght[iloc] <= 0)
    iloc++;
  testErrorRetVA(iloc>=ncomp,mv_outOfBound,"Indice out of bonds %d >= %d ",*err,__LINE__,0,iloc,ncomp);

  logdens_max_loc = mvdens_log_pdf(m->comp[iloc],x,err);
  forwardError(*err,__LINE__,0);
  logdens[iloc] = logdens_max_loc;
  
  for (i=iloc+1;i<ncomp;i++) {
    if (m->wght[i] > 0.0) {
      logdens_tmp = mvdens_log_pdf(m->comp[i],x,err);
      forwardError(*err,__LINE__,0);
      logdens[i] = logdens_tmp;
      if (logdens_max_loc < logdens_tmp) 
        logdens_max_loc=logdens_tmp;
    }
  }
  lpos=0.0;
  for (i=0;i<ncomp;i++) {
    if (m->wght[i] > 0.0) {
      logdens[i] = logdens[i]-logdens_max_loc+DYNMAX;
      lpos = lpos + exp(logdens[i])*m->wght[i];
    }
  }
  lpos = logdens_max_loc+log(lpos)-DYNMAX;
  free(logdens);

  return lpos;
}
#undef __FUNC__

#define __FUNC__ "mix_mvdens_log_pdf_void"
double mix_mvdens_log_pdf_void(const void *m, double *x, error **err)
{
   double res;
   res = mix_mvdens_log_pdf((const mix_mvdens*)m, x, err);
   forwardError(*err, __LINE__, 0);
   return res;
}
#undef __FUNC__

void mvdens_set_band_limit(mvdens *self, size_t value) {
  self->band_limit=value;
}

void mix_mvdens_set_band_limit(mix_mvdens *self, size_t value) {
  size_t i;
  for(i=0;i<self->ncomp;i++)
    self->comp[i]->band_limit=value;
}



